/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.communication.services
 * File:    ServiceHandler.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.communication.services;

import java.security.NoSuchAlgorithmException;

import org.apache.log4j.Logger;
import org.trustedcomputinggroup._2010.ifmap._2.DumpRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.EndSessionType;
import org.trustedcomputinggroup._2010.ifmap._2.ErrorResultType;
import org.trustedcomputinggroup._2010.ifmap._2.NewSessionRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.PollRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.PublishRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.PurgePublisherRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.RenewSessionType;
import org.trustedcomputinggroup._2010.ifmap._2.ResponseType;
import org.trustedcomputinggroup._2010.ifmap._2.SearchRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.SessionRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.SubscribeRequestType;

import de.fhhannover.inform.iron.mapserver.communication.Client;
import de.fhhannover.inform.iron.mapserver.communication.connections.ConnectionContainer;
import de.fhhannover.inform.iron.mapserver.communication.connections.SSRC;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.DumpSessionReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.EndSessionReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.NewSessionReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.PublishReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.PurgePublisherReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.RenewSessionReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.SearchReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.SubscribeReceive;
import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.exceptions.SessionNotFoundException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.IdentifierConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.InvalidMetadataException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.MetadataConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.NoSuchPublisherException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.NoSuchSubscriptionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.ParameterException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.PublisherConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.RequestCreationException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.RunningSessionException;
import de.fhhannover.inform.iron.mapserver.logging.Logging;


/**
 * @author dl, wb
 * 
 * ServiceHandler
 * 
 * created: ???
 *
 * changes:
 *  13.04.2010 - refactor ErrorResponses
 *  10.04.2010 - added bad metadata response
 *  05.03.2010 - added purgePublisher() implementation
 *  04.03.2010 - error responses, cleanup exceptions and notimplemented error
 *  10.02.2010 - added search()
 *  
 *
 */

public class ServiceHandler {
	private static Logger logger;
	static {
		logger = Logging.getTheLogger();
	}
	
	public static ResponseType newSession(NewSessionRequestType newSession, String publisher) {
		try {
			return NewSessionReceive.newSession(newSession, publisher);
		}
		//SessionID is null or length==0 in NewSessionRequest
		catch (RequestCreationException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		//NewSessionRequest is empty in DataModelService
		} catch (ParameterException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		//MD5 is not available
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return getSystemErrorResponse("Internal Error");
		} catch (RunningSessionException e) {
			logger.warn(e.getMessage());
			return getSystemErrorResponse(e.getMessage());
		} catch (PublisherConstructionException e) {
			logger.warn(e.getMessage());
			return getSystemErrorResponse(e.getMessage());
		}
	}

	public static ResponseType renewSession(RenewSessionType renewSessionType){
		try {
			return RenewSessionReceive.renewSession(renewSessionType);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return getSystemErrorResponse("Internal Error");
		} catch (SessionNotFoundException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		}
	}
	
	public static ResponseType endSession(EndSessionType endSessionType) {
		try {
			return EndSessionReceive.endSession(endSessionType);
		} catch (SessionNotFoundException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return getSystemErrorResponse("Internal Error");
		}
	}
	
	public static ResponseType dump(DumpRequestType session) {
		try {
			logger.info("Bearbeite dump()-Anfrage für session '"+session.getSessionId()+"'");
			return DumpSessionReceive.dump(session);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return getSystemErrorResponse("Internal Error");
		} catch (SessionNotFoundException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		} catch (ParameterException e) {
			logger.warn(e.getMessage());
			return getFailureResponse("Internal Error");
		} catch (RunningSessionException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		} catch (PublisherConstructionException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		}		
	}
	
	public static ResponseType publish(PublishRequestType publish) {
		try {
			return PublishReceive.publish(publish);
		} catch (SessionNotFoundException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return getSystemErrorResponse("Internal Error");
		//AccessRequest Name is null or length==0 in AccessRequest
		} catch (IdentifierConstructionException e) {
			logger.warn(e.getMessage());
			return getInvalidIdentifierResponse(e.getMessage());
		//PublisherID, Metadata or Timestamp is null or length==0
		} catch (RequestCreationException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		//Publish is empty in DataModelService
		} catch (ParameterException e) {
			logger.warn(e.getMessage());
			return getFailureResponse("Internal Error");
		} catch (MetadataConstructionException e) {
			logger.warn(e.getMessage());
			return getInvalidMetadataResponse(e.getMessage());
		} catch (InvalidMetadataException e) {
			logger.warn(e.getMessage());
			return getInvalidMetadataResponse(e.getMessage());
		}
	}


	public static ResponseType search(SearchRequestType search) {
		try {
			return SearchReceive.search(search);
		} catch (SessionNotFoundException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return getSystemErrorResponse("Internal Error");
		} catch (IdentifierConstructionException e) {
			logger.warn(e.getMessage());
			return getInvalidIdentifierResponse(e.getMessage());
		} catch (ParameterException e) {
			logger.warn(e.getMessage());
			return getFailureResponse("Internal Error");
		} catch (RequestCreationException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		}
	}
	
	public static ResponseType subscribe(SubscribeRequestType subscribe) {
		try {
			return SubscribeReceive.Subscribe(subscribe);
		} catch (SessionNotFoundException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return getSystemErrorResponse("Internal Error");
		} catch (IdentifierConstructionException e) {
			logger.warn(e.getMessage());
			return getInvalidIdentifierResponse(e.getMessage());
		} catch (ParameterException e) {
			logger.warn(e.getMessage());
			return getFailureResponse("Internal Error");
		} catch (RequestCreationException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		} catch (NoSuchPublisherException e) {
			logger.warn(e.getMessage());
			return getInvalidSessionIDResponse();
		} catch (NoSuchSubscriptionException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		}
		
	}
	
	public static void poll(PollRequestType poll, Client client) {
		/*
		System.err.println("POLL TESTING... Sleeping 360 seconds!!!");
		long starttime = System.currentTimeMillis();
		try {
			Thread.sleep(360000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		long endtime = System.currentTimeMillis();
		return getFailureResponse("Waited: " + (endtime - starttime) + " ms");
		*/
		try {
			SSRC ssrc =ConnectionContainer.getInstance().getSessionBySessionID(poll.getSessionId());
			ssrc.poll(client);
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SessionNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public static ResponseType attachSession(SessionRequestType attachSession) {
		return getFailureResponse("Not inplemented");
	}
	
	public static ResponseType purgePublisher(PurgePublisherRequestType purgePublisher) {
		try {
			return PurgePublisherReceive.purgePublisher(purgePublisher);
		} catch (ParameterException e) {
			logger.warn(e.getMessage());
			return getFailureResponse("Internal Error");
		} catch (RequestCreationException e) {
			logger.warn(e.getMessage());
			return getFailureResponse(e.getMessage());
		}
	}
	
	/**
	 * Private methodes to generate ErrorResults
	 */
	private static ResponseType getAccessDeniedResponse (String errorString){
		return getErrorResponse(errorString, "AccessDenied");
	}
	private static ResponseType getInvalidSessionIDResponse(){
		return getErrorResponse("Session not found", "InvalidSessionID");
	}
	
	private static ResponseType getSystemErrorResponse(String errorString){
		return getErrorResponse(errorString, "SystemError");
	}
	
	private static ResponseType getFailureResponse(String errorString) {
		return getErrorResponse(errorString, "Failure");
	}
	
	private static ResponseType getInvalidMetadataResponse(String errorString) {
		return getErrorResponse(errorString, "InvalidMetadata");
	}
	
	private static ResponseType getInvalidIdentifierResponse (String errorString){
		return getErrorResponse(errorString, "InvalidIdentifier");
	}
	private static ResponseType getInvalidIdentifierTypeResponse (String errorString){
		return getErrorResponse(errorString, "InvalidIdentifierType");
	}
	private static ResponseType getIdentifierTooLongResponse (String errorString){
		return getErrorResponse(errorString, "IdentifierTooLong");
	}
	private static ResponseType getInvalidSchemaVersionResponse (String errorString){
		return getErrorResponse(errorString, "InvalidSchemaVersion");
	}
	private static ResponseType getMetadataTooLongResponse (String errorString){
		return getErrorResponse(errorString, "MetadataTooLong");
	}
	private static ResponseType getSearchResultsTooBigResponse (String errorString){
		return getErrorResponse(errorString, "SearchResultsTooBig");
	}
	private static ResponseType getPollResultsTooBigResponse (String errorString){
		return getErrorResponse(errorString, "PollResultsTooBig");
	}
	
	
	
	private static ResponseType getErrorResponse (String msg, String errorCode){
		ResponseType errorType = new ResponseType();
		ErrorResultType errorResult = new ErrorResultType();
		errorResult.setErrorCode(errorCode);
		errorResult.setErrorString(msg);
		errorType.setErrorResult(errorResult);
		return errorType;
	}
	
}
