/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.configuration.datamodel
 * File:    ConfigurationManager.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.configuration.datamodel;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.configuration.ConfigurationReader;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.IdentityTypeEnum;
import de.fhhannover.inform.iron.mapserver.logging.Logging;

/**
 * A simple configuration manager
 * 
 * FIXME: Should be extended to be used with every possible configuration
 * 	and a properties file. For now just use static variables.
 * 
 * 
 * @author aw, wb
 * @version 0.2
 * 
 * created: 26.11.09
 * changes:
 * 	26.11.09 aw - Created class
 * 	05.12.09 aw - Added defaultPollSize;
 *  17.12.09 wb - useless code added and later removed
 *  07.02.10 aw - removed all those functions and added isIdentityTypeCS(),
 *                added log4j logger,
 *                use properties file,
 *  18.02.10 vp - Added requestPurgeRestrict.
 */
public class ConfigurationManager {
	

	/**
	 * singleton
	 */
	private static ConfigurationManager instance;
	
	private static Logger logger;
	static {
		logger = Logging.getTheLogger();
	}
	
	private String defaultPollSize;
	
	private Map<IdentityTypeEnum, Boolean> identityTypeCSMap;
	
	private boolean administrativeDomainCS;
	
	private ConfigurationReader confReader;
	
	private boolean requestPurgeRestrict;
	
	/**
	 * Private Constructor
	 */
	private ConfigurationManager() {
		confReader = ConfigurationReader.getInstance();
		identityTypeCSMap = new HashMap<IdentityTypeEnum, Boolean>(15);
		
		initCaseSettings();
		initDefaultPollSize();
		initRequestPurgeRestrict();
	}
	
	private void initCaseSettings() {
		String res;
		Boolean value;
		String currentProperty;

		logger.info("Looking up case sensitivity settings...");
		
		// Identity Types
		for (IdentityTypeEnum it : IdentityTypeEnum.values()) {
			currentProperty = "identifier.casesensitive." + it.toString();
			res = confReader.getProperty(currentProperty);
			value = new Boolean(res);
			identityTypeCSMap.put(it, value);
			logger.info("\t" + currentProperty + "=" + value);
		}
		// Administrative Domain
		currentProperty = "identifier.casesensitive.administrativedomain";
		res = confReader.getProperty(currentProperty);
		administrativeDomainCS = new Boolean(res);
		logger.info("\t" + currentProperty + "=" + administrativeDomainCS);
	}
	
	private void initDefaultPollSize() {
		logger.info("Looking up default poll size...");
		String res = confReader.getProperty("defaultpollsize"); 
		try {
		Integer.parseInt(res);
		} catch (NumberFormatException e) {
			logger.warn("Invalid defaultpollsize in configuration: " + res);
			// Could not convert, set default size to a know value
			//FIXME: Undocumented hard coded default value
			res = "10000";
		}
		defaultPollSize = new String(res);
		logger.info("\tdefaultpollsize="+res);
	}
	
	private void initRequestPurgeRestrict() {
		String currentProperty = "request.purge.restrict";
		logger.info("Looking up request purgePublisher restrict...");
		String res = confReader.getProperty(currentProperty); 
		requestPurgeRestrict = new Boolean(res);
		logger.info("\t" + currentProperty + "=" + requestPurgeRestrict);
	}
	
	
	public static ConfigurationManager getInstance() {
		if (instance == null) {
			instance = new ConfigurationManager();
		}
		return instance;
	}
	
	/**
	 * Returns whether this type of identity is casesensitive
	 */
	public boolean isIdentityTypeCaseSensitive(IdentityTypeEnum it) {
		return identityTypeCSMap.get(it);
	}
	
	public String getDefaultPollSize() {
		return new String(defaultPollSize);
		
	}

	public boolean isAdministrativeDomainCS() {
		return administrativeDomainCS;
	}
	
	public boolean isRequestPurgeRestrict() {
		return requestPurgeRestrict;
	}
}
