/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel
 * File:    PublisherRep.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel;

import java.math.BigInteger;
import java.util.HashMap;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.NoSuchPublisherException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.PublisherConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.RunningSessionException;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.clientmapping.Publisher;
import de.fhhannover.inform.iron.mapserver.logging.Logging;

/**
 * Repository which stores Publishers
 * 
 * @author aw
 * @version 0.1
 * 
 */

/*
 * created: 27.11.09
 * changes:
 * 	27.11.09 aw - Created first dummy version to test with api
 * 	05.12.09 aw - Some cleanup and more logic implemented
 *  10.12.09 aw - Reuse publisher objects which have already
 *  	been created, but only the session was unset.
 *  17.12.09 aw - Added max poll size
 *  04.02.10 aw - logging with log4j
 *  07.02.10 aw - added some logging, if we reuse an existing
 *                publisher check for session mapping
 *  10.02.10 aw - removed configureLogging()
 *  11.02.10 vp - the visibility of the classes and methods is changed
 *  27.04.10 aw,vp - throw exceptions if identifier is not available, if
 *                   new session is called and a publisher has a session.
 *  10.04.10 aw - remove singletons!
 *
 */
class PublisherRep {
	
	private static Logger logger;
	
	static {
		logger = Logging.getTheLogger();
	}
	
	PublisherRep() {
		publishers = new HashMap<String, Publisher>();
		sessions = new HashMap<String, Publisher>();
	}
 
	private HashMap<String, Publisher> publishers;
	private HashMap<String, Publisher> sessions;
	 
	/**
	 * Get a publisher with the given publisherId
	 * 
	 * @param id
	 * @return
	 * @throws NoSuchPublisherException if no publisher is available with
	 * the given sessionId
	 */
	Publisher getPublisherByPublisherId(String id) throws NoSuchPublisherException {
		Publisher ret = publishers.get(id);
		if (ret == null) {
			throw new NoSuchPublisherException("No Publisher with PublisherId " + id);
		}
		return publishers.get(id);
	}
	 
	/**
	 * Get a publisher with a given sessionId.
	 * 
	 * @param id
	 * @return
	 * @throws NoSuchPublisherException if no publisher is available with
	 * this a sessionId.
	 */
	Publisher getPublisherBySessionId(String id) throws NoSuchPublisherException {
		Publisher ret = sessions.get(id);
		if (ret == null)
			throw new NoSuchPublisherException("Publisher with SessionId " + id +
					" does not exist");
		return ret;
	}
	 
	/**
	 * Add a Publisher to the repository.
	 * If there already is a publisher with the given
	 * publisherId in the hashmap reuse this publisher
	 * object. If the publisher still has an open
	 * session throw an RunningSessionException
	 * 
	 * @param publisherId
	 * @param sessionId
	 * @return
	 * @throws RunningSessionException
	 * @throws PublisherConstructionException
	 */
	void addPublisher(String publisherId, String sessionId,
													BigInteger maxPollSize) throws
													RunningSessionException,
													PublisherConstructionException {
		
		logger.debug("Adding new Publisher: sessionid=" + sessionId + 
				" publisherid=" + publisherId + " maxpollsize=" + maxPollSize);
		
		Publisher p = publishers.get(publisherId);	
		
		if (p != null) {
			logger.info("Reusing existing publisher object...");
			
			if (p.getSessionId() != null && sessions.containsKey(p.getSessionId())) {
				throw new RunningSessionException("Publisher with PublisherId " + publisherId + " has a open session");
			}
		} else {
			logger.info("Creating new Publisher...");
			p = new Publisher(publisherId, sessionId, maxPollSize);
			publishers.put(publisherId, p);
		}
		p.setSessionId(sessionId);		
		sessions.put(sessionId, p);
	}
	 
	/**
	 * Remove a publisher by its publisher id.
	 * 
	 * @param id
	 */
	void removePublisherByPubliherId(String id) {
		if (id != null) {
			Publisher pub = publishers.get(id);
			if (pub != null) {
				String sessionid = pub.getSessionId();
				if(sessionid != null && sessionid.length() > 0) {
					Publisher pub2 = sessions.get(sessionid);
					if  (pub2 != null) {
						sessions.remove(sessionid);
					}
				}
				publishers.remove(id);
			}
		}
	 
	}

	void removePublisherSession(String sessionId) {
		sessions.remove(sessionId);
	}
}
 
