/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel
 * File:    Searcher.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel;

import java.util.List;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.datamodel.IdentifierRep;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Identifier;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Link;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.meta.Metadata;
import de.fhhannover.inform.iron.mapserver.logging.Logging;
import de.fhhannover.inform.iron.mapserver.messages.SearchRequest;

/**
 * This class does the real searching using a {@link SearchResultBuilder}.
 * 
 * TODO: Are depth checks correct?
 * TODO: Max Size is not checked. We can't interpret {@link Metadata} objects
 * down here, so we don't know the size of them.
 * TODO: terminal-identifier-types is only string comparison, could be improved.
 * TODO: circle searchs can be evil, with unlimited depth...
 * 
 * @author aw
 * @version 0.1
 */

/*
 * created: 09.12.09
 * changes:
 *	09.12.09 aw - Created first version of this class, visibility package
 *		should be enough, prototype search algorithm implemented.
 *	17.12.09 aw - Added terminal-identifier-type return. Should be tested.
 *	17.12.09 aw - terminalIdentifierTypes wasn't set.
 *  04.02.10 aw - added log4j Logger, transformed log calls
 *  05.02.10 aw - use two list which are added into the SearchResult at a
 *                later time
 *  11.02.10 vp - the visibility of the classes and methods is changed
 *  12.02.10 aw - general code cleanup and commenting,
 *                more verbose debugging for the search algorithm
 *                fixed missing depth++ in traverse()
 *  30.04.10 aw - remove singletons!
 *  01.05.10 aw - prototype for Director/Builder pattern, Use a SearchResultBuilder
 *  			  to construct the SearchResult. Don't use two lists and so on...
 *
 */
class Searcher {
	
	private static Logger logger;
	static {
		logger = Logging.getTheLogger();
	}

	/*
	 * The following members can be extracted from the request 
	 */
	private int maxDepth;
	private Identifier startIdent;
	private String terminalIdentifierTypes;
	private Filter matchLinksFilter;
	private Filter resultFilter;
	private IdentifierRep identifierRep;
	
	
	/**
	 * The {@link SearchResultBuilder} to be used.
	 */
	private SearchResultBuilder searchBuilder;
	
	/**
	 * Instantiate a Searcher based upon a SearchRequest
	 * 
	 * @param iRep	reference to the {@link IdentifierRep} instance
	 * @param request the original {@link SearchRequest}
	 * @param builder	{@link SearchResultBuilder} to build the result
	 */
	Searcher(IdentifierRep iRep, SearchRequest request, SearchResultBuilder builder) {
		identifierRep				= iRep;
		maxDepth					= request.getMaxDepth();
		
		startIdent					= request.getStartIdentifier();
		
		searchBuilder				= builder;
		
		matchLinksFilter			= new Filter(request.getMatchLinksFilter());
		resultFilter				= new Filter(request.getResultFilter());
		
		if (request.getTerminalIdentifierTypes() != null) {
			terminalIdentifierTypes = new String(request.getTerminalIdentifierTypes());
		}
	}

	
	/**
	 * Method where the recursion of the search algorithm is started.
	 * Furthermore the result is filtered here.
	 * 
	 * @throws SearchException
	 */
	void runSearch() {
		startIdent = identifierRep.getOrCreateIdentifier(startIdent);
		logger.trace("Beginning search at " + startIdent);
		traverse(startIdent, 0, null);
		searchBuilder.filter(resultFilter);
	}
	
	
	
	/**
	 * This part represents the "real" algorithm.
	 * 
	 * The search algorithm is best looked up in the
	 * Specification. But a short overview is given here:
	 * 
	 * - Add the current {@link Identifier} with its {@link Metadata} objects
	 *   
	 * - Check the depth
	 * 
	 * - Check if the current {@link Identifier} object has a type which
	 *   is in the list of the terminal identifier types
	 *   
	 * - Visit each {@link Link} of the current {@link Identifier} match
	 *   all {@link Metadata} objects against the match links filter.
	 *   
	 * - If any {@link Metadata} object matched the match links filter add this
	 *   {@link Link} object with every matched metadata.
	 *   
	 * - Visit the "other end" of the {@link Link} and begin the
	 *   search algorithm with this {@link Identifier} and a depth
	 *   incremented by one.
	 * 
	 * @param cur
	 * @param depth
	 * @param prev
	 */
	private void traverse(Identifier cur, int depth, Identifier prev) {
		logger.trace("current depth=" + depth);
		
		if (cur == null) {
			logger.fatal("identifier is null, doing simple return");
			return;
		}
		
		appendToResult(cur);
		
		if (depth >= maxDepth) {
			logger.trace("maximum depth reached");
			return;
		}
		
		if (isTerminalIdentifier(cur)) {
			logger.trace(cur + " is a terminal identifier, return");
			return;
		}
		
		List<Link> links = cur.getLinks();
		List<Metadata> mlist = null;
		Identifier next = null;
		
		
		logger.trace("current identfier has " + links.size() + " links");
		for (Link l : links) {
			
			mlist = l.getMetadataInGraph(matchLinksFilter);
			logger.trace("link has n=" + l.getMetadata().size() +
					" with n=" + mlist.size() + " matching");
			
			if (mlist != null && mlist.size() > 0) {
				next = l.getOtherIdent(cur);
				logger.trace("looking at link from " + cur + " to " + next);
				
				if (next == null || next == prev) {
					logger.trace("this is the one we came from, continue...");
					continue;
				}

				// TODO: need information about the already visited identifier
				//if (ires.contains(next)) {
				//logger.warn("Possible circle search, continue...");
				//	continue;
				//}
			
				appendToResult(l, mlist);
				
				logger.trace("traversing to " + next);
				traverse(next, ++depth, cur);
			}
		}
	}
	 

	
	/**
	 * Take this {@link Identifier} and forward it together with it's
	 * Metadata to a {@link SearchResultBuilder}. If its list of Metadata
	 * is empty or contains only Metadata which was not in the graph
	 * then add only the {@link Identifier} to the result.
	 * 
	 * @param cur
	 */
	private void appendToResult(Identifier ident) {
		boolean addedMetadata = false;
		for (Metadata m : ident.getMetadataInGraph()) {
			addedMetadata = true;
			searchBuilder.addMetadata(ident, m);
		}
		
		if (!addedMetadata) {
			searchBuilder.addMetadataContainer(ident);
		}
	}
	
	/**
	 * Same as appendToResult() but for links. In the special case of links
	 * only part of their metadata list is added to the searchresult. This
	 * part must me given in mlist.
	 * 
	 * @param link
	 * @param mlist
	 */
	private void appendToResult(Link link, List<Metadata> mlist) {
		boolean addedMetadata = false;
		for (Metadata m : mlist) {
			addedMetadata = true;
			searchBuilder.addMetadata(link, m);
		}
		if (!addedMetadata) {
			searchBuilder.addMetadataContainer(link);
		}
	}


	/**
	 * Helper method to check whether the given identifier is
	 * a terminal-identifier-type.
	 * 
	 * Realized with contains() from the String class
	 * 
	 * @param cur
	 * @return
	 */
	private boolean isTerminalIdentifier(Identifier cur) {
		logger.trace("current type=" + cur.getType().getTypeString());
		logger.trace("terminal-identifier-types=" + terminalIdentifierTypes);
		
		if (terminalIdentifierTypes != null && 
				terminalIdentifierTypes.contains(cur.getType().getTypeString())) {
			return true;
		}
		return false;
	}
}
 
