/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel.graph.clientmapping
 * File:    Publisher.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel.graph.clientmapping;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.fhhannover.inform.iron.mapserver.datamodel.Subscription;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.PublisherConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.meta.Metadata;

/**
 * This class represents a Publisher which in general represents a MAPC
 * 
 * During a call to newSession() a new Publisher is generated.
 * 
 * A publisher has references to its published metadata. Splitted up into
 * lifetime and session metadata to be easily removed when endSession() or
 * purgePublisher() is called.
 * 
 * 
 * @author aw
 * @version 0.1
 * 
 */

/* 
 * created: 27.11.09
 * changes:
 *  27.11.09 aw - Basic dummy implementation to process some dummy requests
 *                from the api
 *  
 *  05.12.09 aw - Removed the extra session class to be just a String,
 *  	implemented some basic logic.
 *  10.12.09 aw - Added remove functions removeMetadata() removeSessionMetadata()
 *  	and removeLifeTimeMetadata()
 *				- Renamed unsetSession() to deleteSessionMetadata()
 *	17.12.09 vp - Added removeLifeTimeMetadata() and purgeMetadata()
 *	17.12.09 aw - Fixed concurrentException for delete(Session|Lifetime)Metadata()
 *  07.02.10 aw - Added maxPollSize as attribute
 *  08.02.10 aw - toString(), renamed session to sessionId, added some switch cases,
 *                modified to be used without the dangling call from MetadataContainer
 *  10.02.10 aw - removed functions to be more entity like
 *  12.04.10 aw - changed removeMetadata*() to return boolean values
 *  03.05.10 aw, vp - added subscriptions
 *
 */
public class Publisher {
 
	/**
	 * The Publishers ifmap-publisher-id
	 */
	private String publisherId;
	
	/**
	 * The session-id, this can be null if the Publisher currently has no 
	 * session open but still has metadata in the graph
	 */
	private String sessionId;
	
	private List<Metadata> sessionMetadata;
	
	private List<Metadata> lifeTimeMetadata;
	
	private BigInteger maxPollSize;
	
	private Map<String, Subscription> subscriptions;
	
	private boolean notified;

	
	public Publisher(String publisherid, String sessionid,
			BigInteger maxpollsize) throws PublisherConstructionException {
		if (publisherid == null || publisherid.length() == 0)
			throw new PublisherConstructionException("Publisher: " +
					"No publisherID given", publisherid, sessionid);
		
		if (sessionid == null || sessionid.length() == 0)
			throw new PublisherConstructionException("Publisher: " +
					"No publisherID given", publisherid, sessionid);
		
		if (maxpollsize == null)
			throw new PublisherConstructionException("Publisher: " +
					"No maxPollSize given", publisherid, sessionid);
		
		sessionMetadata		= new ArrayList<Metadata>();
		lifeTimeMetadata	= new ArrayList<Metadata>();
		publisherId			= new String(publisherid);
		sessionId			= new String(sessionid);
		maxPollSize			= new BigInteger(maxpollsize.toString());
		subscriptions		= new HashMap<String, Subscription>();
	}
	 
	
	/**
	 * Set the session id of this publisher. Only works if the sessionId
	 * is not null
	 * 
	 * @param sid
	 */
	public void setSessionId(String sid) {
		if (sid != null) {
			sessionId = new String(sid);
		}
	}
	
	/**
	 * The only way to set the sessionId attribute to null.
	 */
	public void deleteSessionId() {
		sessionId = null;
	}
	
	/**
	 * Add a reference to the given metadata. The implementation decides
	 * whether it is added into the session metadata list or lifetime
	 * metadata list.
	 * 
	 * @param m
	 */
	public void addMetadata(Metadata m) {
		if (m != null) {
			switch (m.getLifeTime()) {
			case session:
				addSessionMetadata(m);
				break;
				
			case forever:
				addLifeTimeMetadata(m);
				break;
			}
		}
	}
	
	private void addSessionMetadata(Metadata m) {
		sessionMetadata.add(m);
	}
	
	private void addLifeTimeMetadata(Metadata m) {
		lifeTimeMetadata.add(m);
	}
	
	/**
	 * Return a copy of the session metadata.
	 * 
	 * The metadata objects are not copied!!! Only the list.
	 * 
	 * @return
	 */
	public List<Metadata> getSessionMetadata() {
		return new ArrayList<Metadata>(sessionMetadata);
	}
	 
	public List<Metadata> getLifetimeMetadata() {
		return new ArrayList<Metadata>(lifeTimeMetadata);
	}
	
	public String getPublisherId() {
			return new String(publisherId);
	}
	
	public String getSessionId() {
		if (sessionId != null)
			return new String(sessionId);
		return sessionId;
	}
	
	public BigInteger getMaxPollSize() {
		return new BigInteger(maxPollSize.toString());
	}

	/**
	 * Remove metadata from a publisher.
	 * 
	 * Check which lifetime the Metadata has and call
	 * the corresponding remove methods.
	 * 
	 * @param m
	 */
	public boolean removeMetadata(Metadata m) {
		boolean res = false;
		if (m != null) {
			switch (m.getLifeTime()) {
			case session:
				res = removeSessionMetadata(m);
				break;
				
			case forever:
				res = removeLifeTimeMetadata(m);
				break;
			}
		}
		return res;
	}
	

	private boolean removeLifeTimeMetadata(Metadata m) {
		return lifeTimeMetadata.remove(m);
	}

	private boolean removeSessionMetadata(Metadata m) {
		return sessionMetadata.remove(m);
		
	}
	
	public String toString() {
		return "Publisher<publisherid=" + publisherId + " sessionid=" + sessionId +">";
	}
	
	public Map<String, Subscription> getSubscriptions() {
		return subscriptions;
	}
	
	public void addSubscription(Subscription sub) {
		subscriptions.put(sub.getName(), sub);
	}


	public void clearSubscrptions() {
		subscriptions.clear();
	}
	
	public void setNotified(boolean b) {
		notified = b;
	}
	
	public boolean wasNotified() {
		return notified;
	}
}
 
