/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.utils
 * File:    MessageTransformer.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.utils;


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.trustedcomputinggroup._2010.ifmap._2.DeleteResultType;
import org.trustedcomputinggroup._2010.ifmap._2.DeleteSearchRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.DeleteType;
import org.trustedcomputinggroup._2010.ifmap._2.MetadataListType;
import org.trustedcomputinggroup._2010.ifmap._2.NotifyResultType;
import org.trustedcomputinggroup._2010.ifmap._2.NotifyType;
import org.trustedcomputinggroup._2010.ifmap._2.PollResultType;
import org.trustedcomputinggroup._2010.ifmap._2.PublishRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.ResponseType;
import org.trustedcomputinggroup._2010.ifmap._2.ResultItemType;
import org.trustedcomputinggroup._2010.ifmap._2.SearchRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.SearchResultType;
import org.trustedcomputinggroup._2010.ifmap._2.SearchType;
import org.trustedcomputinggroup._2010.ifmap._2.SubscribeRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.UpdateResultType;
import org.trustedcomputinggroup._2010.ifmap._2.UpdateType;

import de.fhhannover.inform.iron.mapserver.datamodel.Filter;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.IdentifierConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.MetadataConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.RequestCreationException;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.AccessRequest;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Device;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Identifier;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Identity;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.IpAddress;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Link;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.MacAddress;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.meta.MetaLifeTimeType;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.meta.Metadata;
import de.fhhannover.inform.iron.mapserver.messages.PollResult;
import de.fhhannover.inform.iron.mapserver.messages.PublishRequest;
import de.fhhannover.inform.iron.mapserver.messages.RequestFactory;
import de.fhhannover.inform.iron.mapserver.messages.SearchRequest;
import de.fhhannover.inform.iron.mapserver.messages.SearchResult;
import de.fhhannover.inform.iron.mapserver.messages.SubPublishRequest;
import de.fhhannover.inform.iron.mapserver.messages.SubSubscriptionRequest;
import de.fhhannover.inform.iron.mapserver.messages.SubscriptionDelete;
import de.fhhannover.inform.iron.mapserver.messages.SubscriptionRequest;

/**
 * Transform autogenerated messages into datamodel message objects
 * 
 * @author aw, dl
 * 
 * created: 11.02.10
 */


/*
 * 
 * changes:
 *  29.09.10 aw - If a MAP Client does not specify max-depth, the MAP Server MUST
 *                process the search with a max-depth of zero.
 *  11.06.10 aw - optional name possible in searchResults
 *  08.06.10 aw - replace new SubscriptionUpdaet() byte requestFactory call
 *  01.05.10 aw - changed transformSearchResult() to use new SearchResult
 *                implementation.
 *  19.02.10 aw - comments, moved MetadataTransformation in another class
 *  17.02.10 dl - toXMLMetadata
 *  12.02.10 aw - cleanup
 *  10.04.10 aw - throw MetaDataConstructionException upwards,
 *  				send lifetime of metadata into transformation
 *  11.04.10 aw - give timestamp to metadata transformation
 *  27.05.10 Waldemar Bender - create transformSubscribeRequest
 *
 */
public class MessageTransformer {

	private static MessageTransformer instance = new MessageTransformer();
	
	private IdentifierTransformer iTrans = IdentifierTransformer.getInstance();
	private FilterTransformer fTrans = FilterTransformer.getInstance();
	private RequestFactory requestFactory = RequestFactory.getInstance();
	private IdentifierExtractor iExtractor = IdentifierExtractor.getInstance();
	private MetadataTransformer mTrans = MetadataTransformer.getInstance();
	
	public static MessageTransformer getInstance() {
		if (instance == null)
			instance = new MessageTransformer();
		return instance;
	}

	
	/**
	 * 
	 * 
	 * @param search
	 * @return
	 * @throws RequestCreationException
	 * @throws IdentifierConstructionException
	 */
	public SearchRequest transformSearchRequest(SearchType search, String sessionID)
			  throws RequestCreationException, IdentifierConstructionException {
		
		SearchRequest searchRequest = null;
		if (search != null) {
			
			Identifier ident = iExtractor.extractFromSearch((SearchType)search);
// FIXME: Use the transformFilter(String, Map<String, String>) !!!
			Long maxDepth = search.getMaxDepth();
			Long maxSize = search.getMaxSize();
			int maxDepthi = (maxDepth == null) ? 0 : maxDepth.intValue();
			int maxSizei  = (maxSize == null) ? 10000 : maxSize.intValue();
			Filter matchLinks = fTrans.transformFilter(search.getMatchLinks());
			Filter resultFilter = fTrans.transformFilter(search.getResultFilter());
			if (search instanceof SearchRequestType) {
				sessionID = ((SearchRequestType)search).getSessionId();
			}
			
			searchRequest = requestFactory.createSearchRequest(sessionID,
					maxDepthi, maxSizei, search.getTerminalIdentifierType(), ident,
					matchLinks, resultFilter);
		}
		return searchRequest;
	}
	public SearchRequest transformSearchRequest(SearchType search) throws 
							RequestCreationException, IdentifierConstructionException {
		return transformSearchRequest(search, null);
	}	
	
	
	
	public PublishRequest transformPublishRequest(PublishRequestType publish)
					throws IdentifierConstructionException, RequestCreationException,
													MetadataConstructionException{
		PublishRequest publishRequest = null;
		List<Object> list = null;
		List<SubPublishRequest> reqlist = null;
		
		if (publish != null){
			String sid = publish.getSessionId();

			list = publish.getUpdateOrNotifyOrDelete();
			reqlist = transformToSubPublishRequestList(list);
			publishRequest = requestFactory.createPublishRequest(sid, reqlist);
		}
		return publishRequest;
	}
	
	public SubscriptionRequest transformSubscribeRequest (SubscribeRequestType srt) 
						throws RequestCreationException, IdentifierConstructionException {
		
		SubscriptionRequest sr = null;
		List<Object> list = null;
		List<SubSubscriptionRequest> sublist = null;
		
		if (srt != null) {
			String sid = srt.getSessionId();
			list = srt.getUpdateOrDelete();
			sublist = transformToSubSubscriptionList(list, srt.getSessionId());
			sr = requestFactory.createSubscriptionRequest(sid, sublist);
		}
		
		return sr;
	}
	
	

	private List<SubSubscriptionRequest> transformToSubSubscriptionList(
			List<Object> list, String sessionID) 
			throws RequestCreationException, IdentifierConstructionException {
		
		ArrayList<SubSubscriptionRequest> ssr = new ArrayList<SubSubscriptionRequest>();
		for (int i = 0; i < list.size(); i++) {
			if (list.get(i) instanceof DeleteSearchRequestType) {
				ssr.add(new SubscriptionDelete(
						((DeleteSearchRequestType) list.get(i)).getName()));
			} else if (list.get(i) instanceof SubscribeRequestType.Update) {
				SearchRequest sr = transformSearchRequest
							(((SubscribeRequestType.Update)list.get(i)), sessionID);
				
				ssr.add(requestFactory.createSubscriptionUpdate(
						((SubscribeRequestType.Update)list.get(i)).getName(), sr));
			}
		}
		
		return ssr;
	}
	
	


	/**
	 * Transform a datamodel SearchResult to a ResponseType with a set
	 * SearchResultType
	 * 
	 * @param result
	 * @return
	 *
	public ResponseType transformSearchResult(SearchResult result) {
		ResponseType ret = null;
		SearchResultType srt = new SearchResultType();
		
		if (result.getName() != null) {
			srt.setName(result.getName());
		}
		
		if (result != null) {
			for (Identifier i : result.getIdentifiers())  {
				List<Metadata> dmlist = result.getMetadataFor(i);
				ResultItemType rit = transformToResultItem(i, dmlist);
				srt.getResultItem().add(rit);
			}
			
			for (Link l : result.getLinks()) {
				List<Metadata> dmlist = result.getMetadataFor(l);
				ResultItemType rit = transformToResultItem(l, dmlist);
				srt.getResultItem().add(rit);
			}
			
			ret = new ResponseType();
			ret.setSearchResult(srt);
		}
		return ret;
	}
	
	*/
	public ResponseType transformPollResult (PollResult pr) {
		PollResultType prt = new PollResultType();

		// list with all results
		List<Object> list = prt.getSearchResultOrUpdateResultOrDeleteResult();
		// list with delete results
		List<SearchResult> deleteResults =  pr.getDeleteResult();
		List<SearchResult> updateResults =  pr.getUpdateResult();
		List<SearchResult> notifyResults =  pr.getNotifyResult();
		List<SearchResult> searchResults =  pr.getSearchResult();
		
		for (SearchResult sr : deleteResults) {
			list.add(transformDeleteResult(sr));
		}
		
		for (SearchResult sr : updateResults) {
			list.add(transformUpdateResult(sr));
		}
		
		for (SearchResult sr : notifyResults) {
			list.add(transformNotifyResult(sr));
		}

		for (SearchResult sr : searchResults) {
			list.add(transformSearchResult(sr));
		}
		
		
		ResponseType rt = new ResponseType();
		rt.setPollResult(prt);
		
		return rt;
		
	}
	

	private DeleteResultType transformDeleteResult(SearchResult sr) {
		DeleteResultType res = null;
		if (sr != null) {
			res = new DeleteResultType();
			if (sr.getName() != null) {
				res.setName(sr.getName());
			}
			addResultItems(res, sr);
		}
		
		return res;
	}
	

	private UpdateResultType transformUpdateResult(SearchResult sr) {
		UpdateResultType res = null;
		if (sr != null) {
			res = new UpdateResultType();
			if (sr.getName() != null) {
				res.setName(sr.getName());
			}
			addResultItems(res, sr);
		}
		
		return res;
	}
	
	private NotifyResultType transformNotifyResult(SearchResult sr) {
		NotifyResultType res = null;
		if (sr != null) {
			res = new NotifyResultType();
			if (sr.getName() != null) {
				res.setName(sr.getName());
			}
			addResultItems(res, sr);
		}
		
		return res;
	}
	
	public SearchResultType transformSearchResult(SearchResult sr) {
		SearchResultType res = null;
		if (sr != null) {
			res = new SearchResultType();
			if (sr.getName() != null) {
				res.setName(sr.getName());
			}
			addResultItems(res, sr);
		}
		
		return res;
	}


	private void addResultItems(DeleteResultType res, SearchResult result) {
		if (result != null && res != null) {
			List<ResultItemType> rits = createResultItemList(result);
				res.getResultItem().clear();
				res.getResultItem().addAll(rits);
		}
	}


	private void addResultItems(UpdateResultType res, SearchResult result) {
		if (result != null && res != null) {
			List<ResultItemType> rits = createResultItemList(result);
				res.getResultItem().clear();
				res.getResultItem().addAll(rits);
		}
	}
	private void addResultItems(NotifyResultType res, SearchResult result) {
		if (result != null && res != null) {
			List<ResultItemType> rits = createResultItemList(result);
				res.getResultItem().clear();
				res.getResultItem().addAll(rits);
		}
	}
	
	private void addResultItems(SearchResultType res, SearchResult result) {
		if (result != null && res != null) {
			List<ResultItemType> rits = createResultItemList(result);
				res.getResultItem().clear();
				res.getResultItem().addAll(rits);
		}
	}


	private List<ResultItemType> createResultItemList(SearchResult result) {
		List<ResultItemType> list = new LinkedList<ResultItemType>();
		for (Identifier i : result.getIdentifiers())  {
			List<Metadata> dmlist = result.getMetadataFor(i);
			ResultItemType rit = transformToResultItem(i, dmlist);
			list.add(rit);
		}
			
		for (Link l : result.getLinks()) {
			List<Metadata> dmlist = result.getMetadataFor(l);
			ResultItemType rit = transformToResultItem(l, dmlist);
			list.add(rit);
		}
		return list;
	}


	/**
	 * Extract information from a link into a ResultItemType object,
	 * helper mehtod for transformSearchResult()
	 * 
	 * FIXME: Somehow redundant code with the method below
	 * 
	 * @param i
	 * @param items
	 * @param link
	 */
	private ResultItemType transformToResultItem(Link link, List<Metadata> dmlist) {
		ResultItemType rit = null;
		if (link != null) {
			rit = new ResultItemType();
			MetadataListType mlist;
			add(rit, link.getIdent1());
			add(rit, link.getIdent2());
			mlist = mTrans.transformMetadata(dmlist);
			rit.setMetadata(mlist);
		}
		return rit;
	}

	
	/**
	 * Extract information from an identifier and put it into a
	 * ResultItemType object, helper mehtod for transformSearchResult()
	 * 
	 * FIXME: Somehow redundant code with the method above
	 * 
	 * @param item
	 * @param identifier
	 */
	private ResultItemType transformToResultItem(Identifier identifier, List<Metadata> dmlist) {
		ResultItemType rit = null;
		if (identifier != null) {
			rit = new ResultItemType();
			add(rit, identifier);
			MetadataListType mlist = mTrans.transformMetadata(dmlist);
			rit.setMetadata(mlist);
		}
		return rit;
	}

	
	/**
	 * Transform a given identifier into an object of a autogenerated class
	 * and add it into the given ResultItemType object.
	 * 
	 * @param rit
	 * @param identifier
	 */
	private void add(ResultItemType rit, Identifier identifier) {
		if (identifier != null && rit != null) {
			List<Object> retlist = rit.getAccessRequestOrIdentityOrIpAddress();
			if (identifier instanceof AccessRequest) {
				retlist.add(iTrans.transformAR((AccessRequest)identifier));
			} else if (identifier instanceof Device) {
				retlist.add(iTrans.transformDevice((Device)identifier));
			} else if (identifier instanceof Identity) {
				retlist.add(iTrans.transformIdentity((Identity)identifier));
			} else if (identifier instanceof IpAddress) {
				retlist.add(iTrans.transformIP((IpAddress)identifier));
			} else if (identifier instanceof MacAddress) {
				retlist.add(iTrans.transformMacAddress((MacAddress)identifier));
			}
		}
	}


	/**
	 * Wrapper to transform a list of objects which can be of
	 * type UpdateType, DeleteType and NotifyType into
	 * 
	 * 
	 * @param list
	 * @return
	 * @throws IdentifierConstructionException
	 * @throws RequestCreationException
	 * @throws MetadataConstructionException 
	 */
	private List<SubPublishRequest> transformToSubPublishRequestList(List<Object> list)
				throws IdentifierConstructionException, RequestCreationException,
													MetadataConstructionException{
		List<SubPublishRequest> retList = new ArrayList<SubPublishRequest>();
		
		for (Object o : list)
			retList.add(transformToSubPublishRequest(o));
			
		return retList;
	}
	
	

	private SubPublishRequest transformToSubPublishRequest(Object o)
			throws RequestCreationException, IdentifierConstructionException,
												MetadataConstructionException {
		SubPublishRequest ret = null;
		if (o != null) {
			if (o instanceof UpdateType) {
				ret = transformUpdateType((UpdateType)o);
			} else if (o instanceof DeleteType) {
				ret = transformDeleteType((DeleteType)o);
			} else if (o instanceof NotifyType) {
				ret = transformNotifyType((NotifyType)o);
			}
		}
		return ret;
	}

	private SubPublishRequest transformNotifyType(NotifyType value) throws 
									MetadataConstructionException, IdentifierConstructionException, RequestCreationException {
		// FIXME: Quick hack for timestamp
		Date dt = new Date();
		String timestamp = getISO8601DateTime(dt);

		SubPublishRequest ret = null;
		if (value != null) {
			MetaLifeTimeType lt;
			String ltstr = value.getLifetime();
			if (ltstr == null) {
			}
			if (ltstr.equals(MetaLifeTimeType.session.toString())) {
				lt = MetaLifeTimeType.session;
			} else {
				lt = MetaLifeTimeType.forever;
			}
			List<Metadata> mlist = mTrans.transformMetadata(value.getMetadata(), lt, timestamp);
			Identifier idents[] = iExtractor.extractFromNotify(value);
			ret = requestFactory.createPublishNotifyRequest(idents[0], idents[1],
					mlist, lt, timestamp);
		}
		
		return ret;
	}


	private SubPublishRequest transformDeleteType(DeleteType value)
				throws IdentifierConstructionException, RequestCreationException {
		SubPublishRequest ret = null;
	
		if (value != null) {
			Identifier idents[] = iExtractor.extractFromDelete(value);
// FIXME Use new transformFilter()               !!!
			Filter f = fTrans.transformFilter(value.getFilter());
			ret = requestFactory.createPublishDeleteRequest(idents[0], idents[1], f);
		}
		return ret;
	}
	
	

	private SubPublishRequest transformUpdateType(UpdateType value)
				throws IdentifierConstructionException, RequestCreationException,
												MetadataConstructionException {
		
		// FIXME: Quick hack for timestamp
		Date dt = new Date();
		String timestamp = getISO8601DateTime(dt);

		SubPublishRequest ret = null;
		if (value != null) {
			MetaLifeTimeType lt;
			String ltstr = value.getLifetime();
			if (ltstr == null) {
			}
			if (ltstr.equals(MetaLifeTimeType.session.toString())) {
				lt = MetaLifeTimeType.session;
			} else {
				lt = MetaLifeTimeType.forever;
			}
			List<Metadata> mlist = mTrans.transformMetadata(value.getMetadata(), lt, timestamp);
			Identifier idents[] = iExtractor.extractFromUpdate(value);
			ret = requestFactory.createPublishUpdateRequest(idents[0], idents[1],
			mlist, lt, timestamp);
		}
		
		return ret;
	}
	
	protected String getISO8601DateTime(Date dt){
		//FIXME hack in order to get good timestamps for xsd:dateTime
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        SimpleDateFormat tzFormatter = new SimpleDateFormat("Z");
        String timezone = tzFormatter.format(dt);
        return formatter.format(dt) + timezone.substring(0, 3) + ":"
                + timezone.substring(3);
	}
}
